/* -*-c-*-
 *
 * $Id: ghash-def.h,v 1.2 2000/06/17 11:22:03 mdw Exp $
 *
 * Definitions for generic hash interface
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: ghash-def.h,v $
 * Revision 1.2  2000/06/17 11:22:03  mdw
 * Use secure arena for memory allocation.  Minor changes in the generic
 * hash interface.
 *
 * Revision 1.1  1999/12/10 23:21:37  mdw
 * Generic interface.
 *
 */

#ifndef CATACOMB_GHASH_DEF_H
#define CATACOMB_GHASH_DEF_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <mLib/bits.h>
#include <mLib/sub.h>

#ifndef CATACOMB_ARENA_H
#  include "arena.h"
#endif

#ifndef CATACOMB_GHASH_H
#  include "ghash.h"
#endif

/*----- Generic hash function interface -----------------------------------*/

/* --- @GHASH_DEF@ --- *
 *
 * Arguments:	@PRE, pre@ = prefixes for hash function
 *
 * Use:		Defines the generic hash instance.
 */

#define GHASH_DEF(PRE, pre)						\
									\
static const ghash_ops gops;						\
									\
typedef struct gctx {							\
  ghash h;								\
  pre##_ctx c;								\
} gctx;									\
									\
static ghash *ghinit(void)						\
{									\
  gctx *g = S_CREATE(gctx);						\
  g->h.ops = &gops;							\
  pre##_init(&g->c);							\
  return (&g->h);							\
}									\
									\
static void ghhash(ghash *h, const void *p, size_t sz)			\
{									\
  gctx *g = (gctx *)h;							\
  pre##_hash(&g->c, p, sz);						\
}									\
									\
static void ghdone(ghash *h, void *buf)					\
{									\
  gctx *g = (gctx *)h;							\
  pre##_done(&g->c, buf);						\
}									\
									\
static void ghdestroy(ghash *h)						\
{									\
  gctx *g = (gctx *)h;							\
  S_DESTROY(g);								\
}									\
									\
static const ghash_ops gops = { &pre, ghhash, ghdone, ghdestroy };	\
const gchash pre = { #pre, PRE##_HASHSZ, ghinit };

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
